/*
 * Read/write ship data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

SHIPDATA *read_shipdata(int snum)
{
  FILE *handle;
  SHIPDATA *str;

  handle=mkopen("ship.hst",false,'r');
  if (!handle) return NULL;

  if (snum>MAXSHIP || snum==0) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,2+107*(snum-1),SEEK_SET);

  str=new SHIPDATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->wID=readUB2(handle);
  str->wRace=readUB2(handle);
  str->sCode=readXString(handle,3);
  str->wWarp=readUB2(handle);
  str->dis.x=readSB2(handle);
  str->dis.y=readSB2(handle);
  str->pos.x=readUB2(handle);
  str->pos.y=readUB2(handle);
  str->wDriveLevel=readUB2(handle);
  str->wShipType=readUB2(handle);
  str->wBeamLevel=readUB2(handle);
  str->wBeamWeapons=readUB2(handle);
  str->wFighterBays=readUB2(handle);
  str->wTorpLevel=readUB2(handle);
  str->wTorps=readUB2(handle);
  str->wTorpsBays=readUB2(handle);
  str->wMission=readUB2(handle);
  str->wEnemy=readUB2(handle);
  str->wTowed=readUB2(handle);
  str->wDamage=readUB2(handle);
  str->wCrew=readUB2(handle);
  str->cargo.clans=readUB2(handle);
  str->sName=readXString(handle,20);
  str->cargo.fuel=readUB2(handle);
  str->cargo.tri=readUB2(handle);
  str->cargo.dur=readUB2(handle);
  str->cargo.mol=readUB2(handle);
  str->cargo.supplies=readUB2(handle);
  str->toplanet.fuel=readUB2(handle);
  str->toplanet.tri=readUB2(handle);
  str->toplanet.dur=readUB2(handle);
  str->toplanet.mol=readUB2(handle);
  str->toplanet.clans=readUB2(handle);
  str->toplanet.supplies=readUB2(handle);
  str->wPID=readUB2(handle);
  str->toship.fuel=readUB2(handle);
  str->toship.tri=readUB2(handle);
  str->toship.dur=readUB2(handle);
  str->toship.mol=readUB2(handle);
  str->toship.clans=readUB2(handle);
  str->toship.supplies=readUB2(handle);
  str->wSID=readUB2(handle);
  str->wIntercept=readUB2(handle);
  str->cargo.money=readUB2(handle);

  if (extraship) {
    HULLSPEC *hs;
    hs=read_hullspec(str->wShipType);
    str->hullspec=*hs;
    delete hs;
    ENGSPEC *es;
    es=read_engspec(str->wDriveLevel);
    str->engspec=*es;
    delete es;
    if (str->wBeamWeapons > 0) {
      BEAMSPEC *bs;
      bs=read_beamspec(str->wBeamLevel);
      str->beamspec=*bs;
      delete bs;
    }
    if (str->wTorpsBays > 0) {
      TORPSPEC *ts;
      ts=read_torpspec(str->wTorpLevel);
      str->torpspec=*ts;
      delete ts;
    }
  }

  if (str->wRace==0) { // no ship, other data are garbage
    delete str;
    str=NULL;
  }

  fclose(handle);
  return str;
}

bool write_shipdata(int snum,SHIPDATA *str)
{
  FILE *handle;
  bool v;

  handle=mkopen("ship.hst",false,'w');
  if (!handle) return false;

  if (snum>MAXSHIP || snum==0) {
    fclose(handle);
    return false;
  }

  fseek(handle,2+107*(snum-1),SEEK_SET);

  v=writeUB2(handle,str->wID);
  if (v) v=writeUB2(handle,str->wRace);
  if (v) v=writeXString(handle,3,str->sCode);
  if (v) v=writeUB2(handle,str->wWarp);
  if (v) v=writeSB2(handle,str->dis.x);
  if (v) v=writeSB2(handle,str->dis.y);
  if (v) v=writeUB2(handle,str->pos.x);
  if (v) v=writeUB2(handle,str->pos.y);
  if (v) v=writeUB2(handle,str->wDriveLevel);
  if (v) v=writeUB2(handle,str->wShipType);
  if (v) v=writeUB2(handle,str->wBeamLevel);
  if (v) v=writeUB2(handle,str->wBeamWeapons);
  if (v) v=writeUB2(handle,str->wFighterBays);
  if (v) v=writeUB2(handle,str->wTorpLevel);
  if (v) v=writeUB2(handle,str->wTorps);
  if (v) v=writeUB2(handle,str->wTorpsBays);
  if (v) v=writeUB2(handle,str->wMission);
  if (v) v=writeUB2(handle,str->wEnemy);
  if (v) v=writeUB2(handle,str->wTowed);
  if (v) v=writeUB2(handle,str->wDamage);
  if (v) v=writeUB2(handle,str->wCrew);
  if (v) v=writeUB2(handle,str->cargo.clans);
  if (v) v=writeXString(handle,20,str->sName);
  if (v) v=writeUB2(handle,str->cargo.fuel);
  if (v) v=writeUB2(handle,str->cargo.tri);
  if (v) v=writeUB2(handle,str->cargo.dur);
  if (v) v=writeUB2(handle,str->cargo.mol);
  if (v) v=writeUB2(handle,str->cargo.supplies);
  if (v) v=writeUB2(handle,str->toplanet.fuel);
  if (v) v=writeUB2(handle,str->toplanet.tri);
  if (v) v=writeUB2(handle,str->toplanet.dur);
  if (v) v=writeUB2(handle,str->toplanet.mol);
  if (v) v=writeUB2(handle,str->toplanet.clans);
  if (v) v=writeUB2(handle,str->toplanet.supplies);
  if (v) v=writeUB2(handle,str->wPID);
  if (v) v=writeUB2(handle,str->toship.fuel);
  if (v) v=writeUB2(handle,str->toship.tri);
  if (v) v=writeUB2(handle,str->toship.dur);
  if (v) v=writeUB2(handle,str->toship.mol);
  if (v) v=writeUB2(handle,str->toship.clans);
  if (v) v=writeUB2(handle,str->toship.supplies);
  if (v) v=writeUB2(handle,str->wSID);
  if (v) v=writeUB2(handle,str->wIntercept);
  if (v) v=writeUB2(handle,str->cargo.money);

  fclose(handle);
  return v;
}

SHIPDATA * read_all_shipdata()
{
  FILE *handle;
  SHIPDATA *str;
  int snum;
  HULLSPEC *xhullspec=NULL;
  ENGSPEC *xengspec=NULL;
  BEAMSPEC *xbeamspec=NULL;
  TORPSPEC *xtorpspec=NULL;

  handle=mkopen("ship.hst",false,'r');
  if (!handle) return NULL;

  str=new SHIPDATA [501];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  if (extraship) {
    xhullspec=read_all_hullspec();
    xengspec=read_all_engspec();
    xbeamspec=read_all_beamspec();
    xtorpspec=read_all_torpspec();
  }

  fseek(handle,2,SEEK_SET);

  for (snum=1;snum<=500;snum++) {
    str[snum].wID=readUB2(handle);
    str[snum].wRace=readUB2(handle);
    str[snum].sCode=readXString(handle,3);
    str[snum].wWarp=readUB2(handle);
    str[snum].dis.x=readSB2(handle);
    str[snum].dis.y=readSB2(handle);
    str[snum].pos.x=readUB2(handle);
    str[snum].pos.y=readUB2(handle);
    str[snum].wDriveLevel=readUB2(handle);
    str[snum].wShipType=readUB2(handle);
    str[snum].wBeamLevel=readUB2(handle);
    str[snum].wBeamWeapons=readUB2(handle);
    str[snum].wFighterBays=readUB2(handle);
    str[snum].wTorpLevel=readUB2(handle);
    str[snum].wTorps=readUB2(handle);
    str[snum].wTorpsBays=readUB2(handle);
    str[snum].wMission=readUB2(handle);
    str[snum].wEnemy=readUB2(handle);
    str[snum].wTowed=readUB2(handle);
    str[snum].wDamage=readUB2(handle);
    str[snum].wCrew=readUB2(handle);
    str[snum].cargo.clans=readUB2(handle);
    str[snum].sName=readXString(handle,20);
    str[snum].cargo.fuel=readUB2(handle);
    str[snum].cargo.tri=readUB2(handle);
    str[snum].cargo.dur=readUB2(handle);
    str[snum].cargo.mol=readUB2(handle);
    str[snum].cargo.supplies=readUB2(handle);
    str[snum].toplanet.fuel=readUB2(handle);
    str[snum].toplanet.tri=readUB2(handle);
    str[snum].toplanet.dur=readUB2(handle);
    str[snum].toplanet.mol=readUB2(handle);
    str[snum].toplanet.clans=readUB2(handle);
    str[snum].toplanet.supplies=readUB2(handle);
    str[snum].wPID=readUB2(handle);
    str[snum].toship.fuel=readUB2(handle);
    str[snum].toship.tri=readUB2(handle);
    str[snum].toship.dur=readUB2(handle);
    str[snum].toship.mol=readUB2(handle);
    str[snum].toship.clans=readUB2(handle);
    str[snum].toship.supplies=readUB2(handle);
    str[snum].wSID=readUB2(handle);
    str[snum].wIntercept=readUB2(handle);
    str[snum].cargo.money=readUB2(handle);

    if (extraship) {
      str[snum].hullspec=xhullspec[str[snum].wShipType];
      str[snum].engspec=xengspec[str[snum].wDriveLevel];
      if (str[snum].wBeamWeapons > 0) {
        str[snum].beamspec=xbeamspec[str[snum].wBeamLevel];
      }
      if (str[snum].wTorpsBays > 0) {
        str[snum].torpspec=xtorpspec[str[snum].wTorpLevel];
      }
    }
  }

  if (extraship) {
    delete[] xhullspec;
    delete[] xengspec;
    delete[] xbeamspec;
    delete[] xtorpspec;
  }

  fclose(handle);
  return str;
}
