/*
 * Read/write VCR data
 * part of the Host-Toolkit library
 */

#include <stdio.h>
#include "tkhost.h"
#include "tkh.h"
#include "tkiocomm.h"

int num_vcrs()
{
  FILE *handle;
  int ns;

  handle=mkopen("vcr.hst",false,'r');
  if (!handle) return 0;

  ns=(int)readUB2(handle);
  fclose(handle);
  return ns;
}

VCR_DATA *read_vcr_data(int vnum)
{
  FILE *handle;
  VCR_DATA *str;
  int ns;

  handle=mkopen("vcr.hst",true,'r');
  if (!handle) return NULL;

  ns=readUB2(handle);
  if (vnum>ns || vnum<1) {
    fclose(handle);
    return NULL;
  }

  fseek(handle,100*(vnum-1)+2,SEEK_SET);

  str=new VCR_DATA;
  if (!str) {
    fclose(handle);
    return NULL;
  }

  str->RandomSeed=readUB4(handle);
  str->wPlanetTemp=readUB2(handle);
  str->IsPlanet=readUB2(handle);
  for (int k=1;k<3;k++) {
    str->combatant[k].wMass=readUB2(handle);
  }
  for (int k=1;k<3;k++) {
    str->combatant[k].sName=readXString(handle,20);
    str->combatant[k].wDamage=readUB2(handle);
    str->combatant[k].Crew=readUB2(handle);
    str->combatant[k].wID=readUB2(handle);
    str->combatant[k].wRace=readUB2(handle);
    str->combatant[k].wBitmap=readUB2(handle);
    str->combatant[k].wBeamTech=readUB2(handle);
    str->combatant[k].wBeams=readUB2(handle);
    str->combatant[k].wBays=readUB2(handle);
    str->combatant[k].wTorpTech=readUB2(handle);
    str->combatant[k].wTorpFighters=readUB2(handle);
    str->combatant[k].wTorps=readUB2(handle);
  }
  for (int k=1;k<3;k++) {
    str->combatant[k].wShield=readUB2(handle);
  }

  fclose(handle);
  return str;
}

bool write_vcr_data(int vnum,VCR_DATA *str)
{
  FILE *handle;
  bool v;
  int ns;

  handle=mkopen("vcr.hst",false,'w');
  if (!handle) return false;

  ns=readUB2(handle);
  if (vnum>ns || vnum<1) {
    fclose(handle);
    return false;
  }

  fseek(handle,100*(vnum-1)+2,SEEK_SET);

  v=writeUB4(handle,str->RandomSeed);
  if (v) v=writeUB2(handle,str->wPlanetTemp);
  if (v) v=writeUB2(handle,str->IsPlanet);
  for (int k=1;k<3;k++) {
    if (v) v=writeUB2(handle,str->combatant[k].wMass);
  }
  for (int k=1;k<3;k++) {
    if (v) v=writeXString(handle,20,str->combatant[k].sName);
    if (v) v=writeUB2(handle,str->combatant[k].wDamage);
    if (v) v=writeUB2(handle,str->combatant[k].Crew);
    if (v) v=writeUB2(handle,str->combatant[k].wID);
    if (v) v=writeUB2(handle,str->combatant[k].wRace);
    if (v) v=writeUB2(handle,str->combatant[k].wBitmap);
    if (v) v=writeUB2(handle,str->combatant[k].wBeamTech);
    if (v) v=writeUB2(handle,str->combatant[k].wBeams);
    if (v) v=writeUB2(handle,str->combatant[k].wBays);
    if (v) v=writeUB2(handle,str->combatant[k].wTorpTech);
    if (v) v=writeUB2(handle,str->combatant[k].wTorpFighters);
    if (v) v=writeUB2(handle,str->combatant[k].wTorps);
  }
  for (int k=1;k<3;k++) {
    if (v) v=writeUB2(handle,str->combatant[k].wShield);
  }

  fclose(handle);
  return v;
}

VCR_DATA *read_all_vcr_data()
{
  FILE *handle;
  VCR_DATA *str;
  int vnum,vmax;

  vmax=num_vcrs();
  if (vmax==0) return NULL;

  handle=mkopen("vcr.hst",true,'r');
  if (!handle) return NULL;

  fseek(handle,2,SEEK_SET);

  str=new VCR_DATA [vmax+1];
  if (!str) {
    fclose(handle);
    return NULL;
  }

  for(vnum=1;vnum<=vmax;vnum++) {
    str[vnum].RandomSeed=readUB4(handle);
    str[vnum].wPlanetTemp=readUB2(handle);
    str[vnum].IsPlanet=readUB2(handle);
    for (int k=1;k<3;k++) {
      str[vnum].combatant[k].wMass=readUB2(handle);
    }
    for (int k=1;k<3;k++) {
      str[vnum].combatant[k].sName=readXString(handle,20);
      str[vnum].combatant[k].wDamage=readUB2(handle);
      str[vnum].combatant[k].Crew=readUB2(handle);
      str[vnum].combatant[k].wID=readUB2(handle);
      str[vnum].combatant[k].wRace=readUB2(handle);
      str[vnum].combatant[k].wBitmap=readUB2(handle);
      str[vnum].combatant[k].wBeamTech=readUB2(handle);
      str[vnum].combatant[k].wBeams=readUB2(handle);
      str[vnum].combatant[k].wBays=readUB2(handle);
      str[vnum].combatant[k].wTorpTech=readUB2(handle);
      str[vnum].combatant[k].wTorpFighters=readUB2(handle);
      str[vnum].combatant[k].wTorps=readUB2(handle);
    }
    for (int k=1;k<3;k++) {
      str[vnum].combatant[k].wShield=readUB2(handle);
    }
  }

  fclose(handle);
  return str;
}
